#-----------------------------------------------------------------------------
# 
#   ####   #    #  #####    ####           #####   #    #
#  #       #    #  #    #  #               #    #  ##  ##
#   ####   #    #  #####    ####           #    #  # ## #
#       #  #    #  #    #       #   ###    #####   #    #
#  #    #  #    #  #    #  #    #   ###    #       #    #
#   ####    ####   #####    ####    ###    #       #    #
# 
#-----------------------------------------------------------------------------
# Changes:
#-----------------------------------------------------------------------------
# 2012/12/27 BB: In mygrep, I now EXCLUDE lines with # in column 1. 
# 2013/06/27 BB: Changed probleLog to return the log file name
#-----------------------------------------------------------------------------

#------------------------------------------------------------------------------
sub commify
#------------------------------------------------------------------------------
{
  # commify a number. Perl Cookbook, 2.17, p. 64
  my $text = reverse $_[0];
  $text =~ s/(\d\d\d)(?=\d)(?!\d*\.)/$1,/g;
  return scalar reverse $text;
}

#------------------------------------------------------------------------------
sub setEnv
#------------------------------------------------------------------------------
{
  ##&logit("setEnv - Starting"); 

  if (lc $mode eq "test") 
  {
    $dbamonrc_file = "/opt/dbamon/adm/dbamonrc_test";
  }
  else
  {
    $dbamonrc_file = "/opt/dbamon/adm/dbamonrc";
  }

  #chop($admin_email     = `grep ^Admin_EMail: $dbamonrc_file | tail -1 | awk '{ print \$2 }'`) if ! $admin_email;
  #chop($use_oracle      = `grep ^Use_Oracle: $dbamonrc_file | tail -1 | awk '{ print \$2 }'`);
  #chop($use_mssql       = `grep ^Use_MSSQL:  $dbamonrc_file | tail -1 | awk '{ print \$2 }'`);
  #chop($mssql_instance  = `grep ^MSSQL_Instance:  $dbamonrc_file | tail -1 | awk '{ print \$2 }'`);
  #chop($oracle_sid      = `grep ^ORACLE_SID:      $dbamonrc_file | tail -1 | awk '{ print \$2 }'`);

  $admin_email     = &readRCEntry("Admin_EMail:",    $dbamonrc_file ) if ! $admin_email;
  $use_oracle      = &readRCEntry("Use_Oracle:",     $dbamonrc_file );
  $use_mssql       = &readRCEntry("Use_MSSQL:",      $dbamonrc_file );
  $mssql_instance  = &readRCEntry("MSSQL_Instance:", $dbamonrc_file );
  $oracle_sid      = &readRCEntry("ORACLE_SID:",     $dbamonrc_file );

  &error("Either Use_Oracle: or Use_MSSQL: MUST be set to Y in $dbamonrc_file - um=$use_mssql uo=$use_oracle") if $use_mssql ne "Y" && $use_oracle ne "Y";

  if ($use_mssql eq "Y")
  {
    $repos_dbms = "MSSQL"; 
    $repos = "M"; 
    $ENV{"DBAMON_REPOS_DBMS"}     = "MSSQL"; 
    $ENV{"DBAMON_MSSQL_INSTANCE"} = $mssql_instance;
  }
  else
  {
    $repos_dbms = "ORACLE"; 
    $repos = "O"; 
    $ENV{"DBAMON_REPOS_DBMS"} = "ORACLE"; 
    $ENV{"DBAMON_ORACLE_SID"} = $oracle_sid;
  }
  
  ##&logit("DBAMON* Env. vars:"); 
  ##system("env | egrep '^DBAMON' | sort"); 
  ##&logit("setEnv - Done"); 
}

#------------------------------------------------------------------------------
sub msg
#------------------------------------------------------------------------------
{
  my $msgin = $_[0];
  if (length($msgin) > 1000)
  {
    $msgid = substr($msgin,0,1000);
    print "(NOTE: Message text was truncated to 1000 bytes)\n"; 
  }
  #---- Old
  #chop(my $msgdate = `date +%Y/%m/%d-%H:%M:%S-[%a]`);
  #---- New
  $msgdate = &getdate; 

  chop($P = `basename $0`) if $P eq "";
  $verdate = "?" if ! $verdate;
  #my $msgout = sprintf("$msgdate ${P} V$version/$verdate [%5s] | $msgin\n",$$);
  my $pidstring = sprintf("[%5s]",$$);
  $pidstring =~ s/\s/_/g;
  my $msgout = sprintf("$msgdate ${P} $pidstring | $msgin\n",$$);
  print "$msgout";
  ##$diagdata .= $msgout;
  ##$msgsave = $msgin;
}

#------------------------------------------------------------------------------
sub begin 
#------------------------------------------------------------------------------
{
  #---- Set commonly used vars
  $| = 1;
  chop($P = `basename $0`);
  ##$ENV{"PATH"} .= ":$toolsdir" if $toolsdir;
  ##$ENV{"PATH"} .= ":$ENV{'PWD'}" if $toolsdir eq '.';
  $ENV{"PATH"} .= ":/usr/bin:/usr/local/bin:/usr/sbin:/opt/dbamon/bin";
  chop($thisos = `uname`);
  chop($hostname = `hostname`);

  chop($fqhostname = `nslookup $hostname | grep '^Name:' | awk '{ print \$2 }'`) if -e '/etc/resolv.conf' || $g_use_dns eq "Y";
  chop($hostip   = `nslookup $hostname | grep '^Address:' | tail -1 | awk '{ print \$2 }'`) if -e '/etc/resolv.conf' || $g_use_dns eq "Y";

  chop($user     = `whoami`);
  ##chop($date     = `date +%Y/%m/%d-%H:%M:%S-[%a]`);
  ##chop($date     = `date +%Y/%m/%d-%H:%M:%S`);
  $date     = &getdate; 
  $dist_pgmr     = "bbcolo22\@gmail.com";
  ##$dist_rpt      = "bill.border\@hp.com";
  ##$dist_pgmr_pager = "7192441809\@txt.att.net";
  $homedir       = (getpwuid($<))[7];
  $diagdata      = "#---- Started at $date ----\n"; 
  $groupname     = "HP DBA";
  #---- Format incoming arguments
  $i = 0;
  foreach $thiskey (@ARGV)
  {
    $argstring .= "[$i]=$thiskey "; 
    $i++;
  }

  #---- Misc
  ##push(@INC,"/opt/oracle/admin/tools"); 
  ##chdir((getpwuid($<))[7]);  #---- cd to home dir

  $gs_time = time;
  ##$dbatools_event_email = 'oracle@172.18.99.43';  
  ##$dbatools_event_email = 'oracle@tvc301u';  
  ##$dbatools_event_email = 'oracle@ovc323u';  
  $color_red = "780000";
  $color_pink = "lightpink";
  $color_lime = "99ff99";
  $color_yellow = "ffff99";

  #---- Print startup message 
  ##&msg("$P Starting - Version $version of $verdate - Arguments: $argstring"); 
  &msg("$P Starting - Arguments=($argstring) hostname=$hostname user=$user fqhostname=$fqhostname PWD=$ENV{'PWD'}") if $debug; 
}

#-----------------------------------------------------------------------------
sub parse
#-----------------------------------------------------------------------------
{
  ##print "before: $_";
  s/\\/\\\\/g;   # Change all backslashes to double backslashes
  ##print "after: $_";
  #---- 2011/10/03 BB - Get rid of control-M crud
  s/\x0d//g;  
  
  $F0 = $_;
  @words = split;
  $F1 = $F2 = $F3 = $F4 = $F5 = $F6 = $F7 = $F8 = $F9 = $F10 = $F11 = $F12 = $F13 = $F14 = $F15 = "";
  $NF = 0;
  if (substr($words[0],0,1) eq "#")
  {
    return;
  }
  for ($parse_i = 0; $parse_i <= $#words; $parse_i++)
  {
    $parse_z = $parse_i + 1;  # AWK array is relative to 1, not 0
    $words[$parse_i] =~ s/\@/\\\@/;
    $words[$parse_i] =~ s/\$/\\\$/;
    $words[$parse_i] =~ s/"/'/g;
    $stmt = "\$F${parse_z} = \"$words[$parse_i]\"";
    ##print "stmt=$stmt \n";
    eval($stmt);
  }
  $NF = $#words + 1;

  #---- Extract F_noun
  $F_noun =  (split(/\:/,$F0)) [1];   #---- Everything to the right of the COLON 
  $F_noun =~ s/^\s+//g;   #---- Remove all LEADING whitespace
  $F_noun =~ s/\s+$//g;   #---- Remove all TRAILING whitespace

  ##print "DEBUG-PARSE - F_noun = ($F_noun)\n"; 

  return;
}

#-----------------------------------------------------------------------------
sub getdate
#-----------------------------------------------------------------------------
{
  my ($sec,$min,$hour,$mday,$mon,$year) = localtime(time);
  $year += 1900;
  $mon += 1;

  # Pad with 0's
  $yy = sprintf("%04d",$year);
  $mm = sprintf("%02d",$mon);
  $dd = sprintf("%02d",$mday);
  $hh = sprintf("%02d",$hour);
  $mi = sprintf("%02d",$min);
  $ss = sprintf("%02d",$sec);

  my $dw = "$yy/$mm/$dd-$hh:$mi:$ss";
  return $dw;
}

#------------------------------------------------------------------------------
sub timeToText
#------------------------------------------------------------------------------
{
  my $zz_ts = $_[0];
  my ($sec,$min,$hour,$mday,$mon,$year) = localtime($zz_ts);
  $year += 1900;
  $mon += 1;

  # Pad with 0's
  my $yy = sprintf("%04d",$year);
  my $mm = sprintf("%02d",$mon);
  my $dd = sprintf("%02d",$mday);
  my $hh = sprintf("%02d",$hour);
  my $mi = sprintf("%02d",$min);
  my $ss = sprintf("%02d",$sec);

  my $date = "$yy/$mm/$dd-$hh:$mi:$ss";
  return $date;
}

#------------------------------------------------------------------------------
sub timeToNum
#------------------------------------------------------------------------------
{
  use Time::Local;
  my $zz_text = $_[0];
  $zz_text =~ s/-/\//g;
  $zz_text =~ s/:/\//g;
  ##print "zz_text=$zz_text\n";
  @words = split(/\//,$zz_text);

  if ($#words != 5)
  {
    &error("timeToNum() Input timestamp ($zz_text) should be in the format: YYYY/MM/DD-HH:MM:SS");
  }

  my $zz_yyyy = $words[0];
  my $zz_mon  = $words[1];
  my $zz_dd   = $words[2];
  my $zz_hh   = $words[3];
  my $zz_mm   = $words[4];
  my $zz_ss   = $words[5];
  ##print "zz_yyyy=$zz_yyyy zz_mon=$zz_mon zz_dd=$zz_dd zz_hh=$zz_hh zz_mm=$zz_mm zz_ss=$zz_ss\n";

  &error("timeToNum() - ($zz_text) YYYY must be 1950-2099") if ($zz_yyyy < 1950 || $zz_yyyy > 2099);
  &error("timeToNum() - ($zz_text) MON  must be 1-12     ") if ($zz_mon  <    1 || $zz_mon  >   12);
  &error("timeToNum() - ($zz_text) DD   must be 1-31     ") if ($zz_dd   <    1 || $zz_dd   >   31);
  &error("timeToNum() - ($zz_text) HH   must be 0-23     ") if ($zz_hh   <    0 || $zz_hh   >   23);
  &error("timeToNum() - ($zz_text) MM   must be 0-59     ") if ($zz_mm   <    0 || $zz_mm   >   59);
  &error("timeToNum() - ($zz_text) SS   must be 0-59     ") if ($zz_ss   <    0 || $zz_ss   >   59);

  @zz_date=($zz_ss,$zz_mm,$zz_hh,$zz_dd,--$zz_mon,$zz_yyyy);
  $zz_result = &timelocal(@zz_date);

  return $zz_result;
}

#------------------------------------------------------------------------------
sub packetParse()
#------------------------------------------------------------------------------
{
  $in_packet = $_[0];
  
  $pa_debug = 0; 
  &msg("DEBUG_PA: in_packet=$in_packet ") if $pa_debug;

  $pa_delim = "<|>";
  undef @pa_words;
  undef @pa_offsets;

  $pa_key  = (split(/$pa_delim/,$in_packet)) [0];
  $pa_pgm  = (split(/$pa_delim/,$in_packet)) [1];
  $pa_verb = (split(/$pa_delim/,$in_packet)) [2];
  $pa_d1   = (split(/$pa_delim/,$in_packet)) [3];

  for ($i=0; $i <= length($in_packet); $i++)
  {
    ##&msg("i=$i");
    if (substr($in_packet,$i,length($pa_delim)) eq $pa_delim)
    {
      &msg("HIT: $i") if $pa_debug;
      push (@pa_offsets, $i + length($pa_delim));
    }
  }

  $pa_counter = -1;
  foreach $pa_this (@pa_offsets)
  {
    $pa_counter++;
    $pa_next = $pa_offsets[$pa_counter+1] - $pa_this - length($pa_delim);
    last if $pa_next < 0;
    print "pa_this=$pa_this pa_next=$pa_next\n" if $pa_debug;
    push(@pa_words, substr($in_packet,$pa_this,$pa_next));
  }

  if ($pa_debug)
  {
    $pa_counter = -1;
    foreach (@pa_words)
    {
      $pa_counter++;
      print "DEBUG_PA: pa_words[$pa_counter]=($_)\n";
    }
  }
}

#----------------------------------------------------------------------
sub readRCEntry
#----------------------------------------------------------------------
{
  $my_verb     = $_[0];
  $my_dbamonrc = $_[1];

  $my_dbamonrc = "/opt/dbamon/adm/dbamonrc" if ! $my_dbamonrc;
  $my_rval = "";
  $my_lines = 0;

  $debug_rc = 0;

  &msg("readRC - Looking for dbamonrc value for $my_verb - my_dbamonrc=$my_dbamonrc") if $debug_rc;
  ##open(RC,"cat $my_dbamonrc|") || &error("Cannot open input $my_dbamonrc - $!");
  close(RC); 
  open(RC,"$my_dbamonrc") || &error("Cannot open input $my_dbamonrc - $!");
  while(<RC>)
  {
    next if /^#/;
    next if /^$/;
    $my_lines++; 
    &msg("LINE: $_") if $debug_rc;
    @my_words = split;
    if (uc $my_words[0] eq uc $my_verb) { $my_rval = $my_words[1]; last; }
  }
  close(RC);
  &msg("readRC - my_lines=$my_lines Returning: $my_rval") if $debug_rc;
  return $my_rval;
}

#----------------------------------------------------------------------
sub probeLog
#----------------------------------------------------------------------
{
  my $cp_method = "P"; 
  my $cp_file   = "$tmpdir/debug/probeout_${mode}_${this_function}_${titleraw[$thishost]}.txt";

  if ($cp_method eq "P") 
  {
    use File::Copy;
    copy("$tmpfl3", $cp_file);
  }
  else
  {
    system("cp $tmpfl3 $cp_file");
  } 

  return $cp_file; 
}

#------------------------------------------------------------------------------
sub pageHeader
#------------------------------------------------------------------------------
{
  my $ph_title   = $_[0];
  my $ph_refresh = $_[1];

  my $ph_title_font = "verdana";
  my $ph_table_width = "900";
  my $mode_ucfirst = ucfirst $mode; 

  $ph_refresh = 86400 if ! $ph_refresh;

  #---- Sizes were 48/15 - Now 30/12
  my $ph_content = "<title>DBAmon $ph_title - Mode: $mode_ucfirst</title>
<meta http-equiv=Refresh content=$ph_refresh>
<link REL='SHORTCUT ICON' href='/dbamon/img/favicon.ico'>
<body bgcolor=white link=darkblue vlink=darkblue>
<font face=$font>
<table width=$ph_table_width bgcolor=wheat>
<tr bgcolor=darkblue>
  <td align=center valign=center style='font-family:$ph_title_font;font-size:30;color:white;'>DBAmon
<tr bgcolor=wheat>
  <td>
    <span style='font-family:$ph_title_font; font-size:12pt;'>
    <center>$ph_title</center>
</table>
  ";

  #---- $links
  #---- <hr>

  return $ph_content;
}

#------------------------------------------------------------------------------
sub mygrep
#------------------------------------------------------------------------------
{
  my $mg_string  = $_[0];
  my $mg_inpath  = $_[1];
  my $mg_outpath = $_[2];
  
  my $mg_hits = 0; 
  my $mg_outdata = ""; 

  return -1 if ! -e "$mg_inpath"; 

  open(MGIN,"$mg_inpath") || &error("Cannot open input mg_inpath=$mg_inpath"); 
  while(<MGIN>) 
  {
    next if /^#/;

    if (/$mg_string/) 
    {
      $mg_hits++; 
      if ($mg_outpath) 
      { 
        $mg_outdata .= $_; 
      }
    }
  }
  close(MGIN); 

  if ($mg_outdata) 
  {
    open(MGOUT,">$mg_outpath") || &error("Cannot open output mg_outpath=$mg_outpath"); 
    print MGOUT $mg_outdata; 
    close(MGOUT); 
  }

  return $mg_hits;
}

#------------------------------------------------------------------------------
sub fileDetails
#------------------------------------------------------------------------------
{
  my $fd_file = $_[0];
  
  my $fd_rows  = 0;
  my $fd_bytes = 0;

  return -1, -1 if ! -e "$fd_file"; 

  open(FD,"$fd_file") || &error("L428 Cannot open input fd_file=$fd_file"); 
  while(<FD>) 
  {
    $fd_rows++; 
    $fd_bytes += length($_); 
  }
  close(FD); 

  return $fd_rows, $fd_bytes;
}

#------------------------------------------------------------------------------
sub fileReadScalar
#------------------------------------------------------------------------------
{
  my $fd_file = $_[0];
  
  my $fd_out   = ""; 

  return "" if ! -e "$fd_file"; 

  open(FD,"$fd_file") || &error("L449 Cannot open input fd_file=$fd_file"); 
  while(<FD>) 
  {
    $fd_out .= $_; 
  }
  close(FD); 

  return $fd_out;
}

#------------------------------------------------------------------------------
sub fileReadArray
#------------------------------------------------------------------------------
{
  my $fd_file = $_[0];
  
  my @fd_out   = (); 

  return "" if ! -e "$fd_file"; 

  open(FD,"$fd_file") || &error("L449 Cannot open input fd_file=$fd_file"); 
  while(<FD>) 
  {
    push(@fd_out, $_); 
  }
  close(FD); 

  return @fd_out;
}

#------------------------------------------------------------------------------
sub cksum
#------------------------------------------------------------------------------
{
  my $ck_string_in = $_[0];

  my $ck_result = "";
  my $ck_str = "";

  use Digest::MD5 qw(md5);
  $ck_str = substr( md5($ck_string_in ), 0, 4 );
  $ck_result = sprintf unpack('L', $ck_str);  # Convert to 4-byte integer (long)

  return $ck_result;
}

#------------------------------------------------------------------------------
sub nounFromString
#------------------------------------------------------------------------------
{
  my ($nf_searchfor, $nf_string) = @_;

  my $nf_return = "";
  my $nf_debug  = 0;

  my $zl      = -999999;
  my $zx1     = -999999;
  my $zx2     = -999999;
  my $zxstart = -999999;
  my $zxlen   = -999999;

  print "DEBUG - nf_searchfor=$nf_searchfor\n" if $nf_debug;
  print "DEBUG - nf_string=$nf_string\n" if $nf_debug;

  $zl  = length($nf_searchfor);
  $zx1 = index($nf_string, $nf_searchfor);
  $zx2 = index($nf_string, " ", $zx1 + $zl);

  print "DEBUG - zx1=$zx1\n" if $nf_debug;
  print "DEBUG - zx2=$zx2\n" if $nf_debug;
  print "DEBUG - zl=$zl\n" if $nf_debug;

  $zxstart = $zx1 + $zl;
  $zxlen   = $zx2 - $zxstart;

  print "DEBUG - zxstart=$zxstart\n" if $nf_debug;
  print "DEBUG - zxlen  =$zxlen\n" if $nf_debug;

  if ($zxlen > 0)
  {
    $nf_return = substr($nf_string, $zxstart, $zxlen);
  }
  else
  {
    $nf_return = substr($nf_string, $zxstart);
  }

  print "DEBUG - nf_return=($nf_return)\n" if $nf_debug;

  return $nf_return;
}

#------------------------------------------------------------------------------
sub phoneHome
#------------------------------------------------------------------------------
{
  my ($ph_thishost, $ph_subject, $ph_file) = @_;

  if ($ph_thishost > 0 && $ph_file) 
  {
    system("cat $ph_file | dbamon_mailer -s 'DBAmon | $titleraw[$ph_thishost] | phoneHome: $ph_subject' $admin_email");
  }
  elsif ($ph_thishost > 0) 
  {
    system("echo ' ' | dbamon_mailer -s 'DBAmon | $titleraw[$ph_thishost] | phoneHome: $ph_subject' $admin_email");
  }
  elsif ($ph_file > 0) 
  {
    system("cat $ph_file | dbamon_mailer -s 'DBAmon | phoneHome: $ph_subject' $admin_email");
  }
  else 
  {
    system("echo ' ' | dbamon_mailer -s 'DBAmon | phoneHome: $ph_subject' $admin_email");
  }
}

#------------------------------------------------------------------------------
sub serializeMe
#------------------------------------------------------------------------------
{
  use Fcntl qw(:DEFAULT :flock);
  $ser_debug = 1;
  $ser_dir   = "/tmp/ser_lockfiles";
  &error("Var P is not set") if ! $P;
  ($ser_p = $P) =~ s/.pl//;
  $ser_file  = "$ser_dir/${ser_p}.txt";
  $ser_ts    = &getdate;

  #---- Create lock dir if it doesn't already exist
  mkdir $ser_dir if ! -e $ser_dir;

  &msg("serializeMe - Starting - ser_p=$ser_p ser_dir=$ser_dir ser_file=$ser_file") if $ser_debug;

  #---- Open output lockfile
  &msg("ser - Opening file ser_file=$ser_file - Lock will follow") if $ser_debug;
  open(LOCKFILE,">>$ser_file") || &error("Cannot open output append ser_file=$ser_file - $!");
  print LOCKFILE "$ser_ts - $P - $$\n";

  $! = "";
  $flock_rc = -1;
  $flock_rc = flock(LOCKFILE, LOCK_EX|LOCK_NB);
  &msg("ser - After flock - flock_rc=$flock_rc bang=$!");

  #---- Evaluate results
  if ($flock_rc == 0)
  {
    my $dashes = '-' x 80;
    &msg($dashes);
    &msg("NOTE: Another iteration of $P must be running");
    &msg("  Cannot lock file (flock_rc=$flock_rc !=$!)");
    &msg("  Exiting 0");
    &msg($dashes);
    exit 0;
  }

  &msg("serializeMe - Done") if $ser_debug;
}

#----------------------------------------------------------------------
1;
